==> sage-v0.3-2005-05-02-src.tar: sage-0.3/sage/rings/real_number.py <==
"""
Real number

Real numbers in MANIN are represented either as GMP MPF's or 
using the Python Decimal type.  
"""


import gmpy
import math

import coerce
import decimal
import element
import infinity
import sage.misc.misc as misc
import operator
import sage.pari.wrap
import sage.rings.rational as rational
import sage.rings.integer as integer
import real_field
import sage.rings.polynomial_ring
import integer_ring

class RealNumber(element.Element):
    """
    Abstract real number base class.
    """
    def __init__(self, cls):
        self.__cls = cls

    def __abs__(self):
        if self < 0:
            return -self
        return self

    def __add__(self, right):
        """
        Return self + right.
        EXAMPLES:
            >>> RealNumber_mpf('2/3') + RealNumber_decimal(0.333)
            0.999666666666666666667
        """
        if not isinstance(right, self.__cls):
            return coerce.bin_op(self, right, operator.add)
        return self._new(self.value + right.value)
    
    def __cmp__(self, other):
        """
        Compare self and other.
        
        EXAMPLES:
        In comparisons numeric types, e.g., int, long, float, and those derived
        from Element, are converted to a RealNumber (if it is on the left), 
        to the precision of the number on the left.
            >>> R = RealNumber_mpf
            >>> R('2/3') < 1
            True
            >>> R('1/3') > 0.1
            True
            >>> R('1/3') > RealNumber_decimal(0.1)
            True
            >>> R('1/3') == R('0.3333333333333333')
            False
            
        Strings are not numeric, so they are not converted.
            >>> R(1) == '1'
            False
        """
        if not isinstance(other, self.__cls):
            if isinstance(other, (int, long, float, complex, element.Element)):
                other = self._new(other, self.prec())
            else:
                return -1
        return misc.generic_cmp(self.value, other.value)

    def __div__(self, right):
        """
        Return the quotient self / right.
        
        EXAMPLES:
            >>> R = RealField_mpf()
            >>> R(1) / R(3)
            0.333333333333333333333
        """
        if not isinstance(right, self.__cls):
            return coerce.bin_op(self, right, operator.div)
        return self._new(self.value / right.value)
        
    def __float__(self):
        """
        Return the coercion of self to a Python float.
        
        EXAMPLES:
            >>> R = RealField_mpf()
            
            >> pi = R.pi(200); pi
            3.14159265358979323846264338327950288419716939937510582097494459230782
            >> float(pi)
            3.1415926535897931
        """
        return float(self.value)
    
    def __int__(self):
        """
        Return the coercion of self to a Python int.  This is the integer
        part of self.
        
        EXAMPLES:
            >>> R = RealField_mpf()
            
            >> pi = R.pi(200); pi
            3.14159265358979323846264338327950288419716939937510582097494459230782
            >> int(pi)
            3
            
        If the real number is too large to fit in a Python int, it is 
        coerced to a Python long (note the L below):
            >>> int(R(141592653589793238462643383279502))
            141592653589793238462643383279502L
            
        Note that int is not "rounding".  It simply takes the integer part
        of the decimal expansion, including the sign:
            >>> int(R(-0.8))
            0
            >>> int(R(0.8))
            0
            >>> int(R(-2.3))
            -2
        """
        try:
            return int(self.value)
        except OverflowError:
            return long(self.value)
            
    def __invert__(self):
        """
        Return the inverse of self.
        
        EXAMPLES:
            >>> R = RealField_mpf()
            
            >> pi = R.pi(); pi
            3.14159265358979323846
            >> ~pi    # the additive inverse
            0.318309886183790671538
            
        Division by zero raises a ....
            >>> ~R(0)
            Traceback (most recent call last):
            ...
            ZeroDivisionError: inverse of 0 not defined
        """
        try:
            return self._new(1/self.value)
        except ZeroDivisionError:
            raise ZeroDivisionError, "inverse of 0 not defined"
            
    def __long__(self):
        """
        Return the coercion of self to a Python long.
        
        EXAMPLES:
            >>> R = RealField_mpf()
            
            >> pi = R.pi(200); pi
            3.14159265358979323846264338327950288419716939937510582097494459230782
            >> long(pi)
            3L
            
        Note that int is not "rounding".  It simply takes the integer part
        of the decimal expansion, including the sign:
            >>> long(R(-0.8))
            0L
            >>> long(R(0.8))
            0L
            >>> long(R(-2.3))
            -2L
        """
        return long(self.value)
    
    
    def __mul__(self, right):
        """
        Return the product of self and right.
        
        If right is not a RealNumber, an attempt is made to coerce self
        and right into a common parent.
        
        EXAMPLES:
            >>> R = RealField_mpf()
            >>> pi = R.pi()
            >>> A = pi * 25; A
            78.5398163397448309615
            >>> isinstance(A, real_number.RealNumber_mpf)
            True

        In this example, a string is automatically coerced to a real number.
            >>> pi * '2.01'
            6.31460123371548440927
            
        Of course two strings don't coerce to RealNumber's:
            >>> '3.14' * '2.01'
            Traceback (most recent call last):
            ...
            TypeError: can't multiply sequence by non-int
        """
        if not isinstance(right, self.__cls):
            return coerce.bin_op(self, right, operator.mul)
        return self._new(self.value * right.value)
    
    def __neg__(self):
        """
        Return the negative of self.
        
        EXAMPLES:
            >>> x = RealNumber_mpf('1.23456')
            >>> -x
            -1.23456
        """
        return self._new(-self.value)
    
    def __repr__(self):
        return "%s"%self.value


    def __radd__(self, left):
        return  self._new(left) + self
    
    def __sub__(self, right):
        if not isinstance(right, self.__cls):
            return coerce.bin_op(self, right, operator.sub)
        return self._new(self.value - right.value)
    
    def __rsub__(self, left):
        return self._new(left) - self

    def __rmul__(self, left):
        return self._new(left)*self
    
    def __rdiv__(self, left):
        return self._new(left)/self
    
    def __pow__(self, right):
        return self._new(self.value ** right)

    def __pos__(self):
        return self
  
    def _new(self, value, prec=0):
        raise NotImplementedError

    def arg(self):
        return 0

    def real(self):
        return self

    def imag(self):
        return self.parent()(0)

    def log(self):
        """
        Return the logarithm of self.

        IMPORTANT WARNING: Currently the computation is done by
        converting the number to a float and using the standard
        library function.  Thus there is precision loss!
        """
        z = math.log(float(self.value))
        return self.parent()(z)
  
    def order(self):
        if self.value == 1:
            return 1
        if self.value == -1:
            return 2
        return infinity.infinity
        
    def prec(self):
        raise NotImplementedError

    def __get_pari_object(self):
        try:
            return self.__pari_object
        except AttributeError:
            self.__pari_object = sage.pari.wrap.PariObject(str(self))
            return self.__pari_object

    pari_object = property(__get_pari_object, \
                           doc = "PARI real associated to this real number")

    def algebraic_dependence(self, n, flag=0):
        """
        Try to find a polynomial of degree at most n having this
        real number x as an approximate root.

        The polynomial that is obtained is not necessarily
        "correct", and it's not even guaranteed to be irreducible.
        One can check the closeness either by evaluation or by
        computing the roots of the polynomial given by algdep.

        INPUT:
            n -- positive integer
            flag -- 0 (default): Use a variant of the LLL algorithm
                       due to Hastard, Lagarias, and Schnorr (STACS
                       1986).  If the precision is too low, the
                       routine may enter an infinite loop.
                    >0: Use a standard LLL algorithm, where the
                       flag is a precision, which should be between
                       0.5 and 1.0 times the number of decimal digits
                       to which x was computed.
        OUTPUT:
            A polynomial over the integers.

        IMPLEMENTATION: Call PARI's algdep function.  Note that the
            default real precision of PARI is used, not the precision
            of this real number.

        EXAMPLE:
        We compute the minimal polynomial of sqrt(2):
        
            >>> import real_field, math
            >>> R = real_field.RealField()
            >>> a = R(math.sqrt(2))
            >>> a.algebraic_dependence(2)
            x^2 - 2

        Here we compute the real number 123/456, which is not in lowest terms,
        and find the minimal polynomial, which is a polynomial of degree 1.
            >>> a = R('123/456')
            >>> a
            0.269736842105263157895
            >>> a.algebraic_dependence(1)
            152*x - 41
            >>> 41/152.0
            0.26973684210526316
        """
        f = self.pari_object.algdep(n, flag)
        v = f.Vec().list()
        v.reverse()
        R = sage.rings.polynomial_ring.PolynomialRing(integer_ring.IntegerRing())
        g = R(v)
        return g
  
  
class RealNumber_decimal(RealNumber):
    """
    A real number with an arbitrary number of bits of precision.
    This is implemented using the builtin Python Decimal type.

    Create a number using
        RealNumber_decimal(value,)

    INPUT:
        value -- a float, integer, or string
    EXAMPLES:
        >>> R = real_number.RealNumber_decimal
        >>> R(1.32)
        1.32
        >>> R('2/3')
        0.6666666666666666666666666667
        >>> x = R(1.234); y = R(-1.1)
        >>> x < y
        False
        >>> x > y
        True
        >>> x + y
        0.134
        >>> x - y
        2.334
        >>> x * y
        -1.3574
        >>> x / y
        -1.121818181818181818181818182
        >>> x*100
        123.400
        >>> x**100
        1353679866.791074518495392801
        >>> 2*x
        2.468
        >>> x*2.0
        2.4680
        >>> x.order()
        Infinity
        >>> x.prec()
        Infinity
        >>> float(x)
        1.234
        >>> int(y)
        -1
    """
    def __init__(self, value):
        element.Element.__init__(self, real_field.RR_decimal)
        RealNumber.__init__(self, RealNumber_decimal)
        try:
            if isinstance(value, decimal.Decimal):
                self.value = decimal.Decimal(value)
            elif isinstance(value, RealNumber_decimal):
                self.value = decimal.Decimal(value.value)
            elif isinstance(value, rational.Rational):
                self.value = decimal.Decimal(str(value.numerator())) / \
                                decimal.Decimal(str(value.denominator()))
                return
            elif isinstance(value, str):
                i = value.find("/")
                if i != -1:
                    self.value = decimal.Decimal(value[:i]) / \
                                    decimal.Decimal(value[i+1:])
                    return
                self.value = decimal.Decimal(value)
                return
            else:
                self.value = decimal.Decimal(str(value))
        except TypeError:
            raise TypeError, "unable to coerce %s to a RealNumber_decimal"%value
        
    def _new(self, value, prec=0):
        return RealNumber_decimal(value)
  
    def prec(self):
        return infinity.infinity
    
 
class RealNumber_mpf(RealNumber):
    """
    A real number with a fixed number of bits of precision.

    Create a number using
         RealNumber_mpf(value, bits=0)

    INPUT:
        value -- a float, integer, or string
        prec -- an int, lower bound on the number of bits of precision
                (always >= 64).
    EXAMPLES:
        >>> R = real_number.RealNumber_mpf
        >>> x = R(1.234); y = R(-1.1)
        >>> x < y
        False
        >>> x > y
        True
        >>> x + y
        0.133999999999999896971
        >>> x - y
        2.33400000000000007461
        >>> x * y
        -1.35740000000000009397
        >>> x / y
        -1.12181818181818171468
        >>> x*100
        123.399999999999998579
        >>> x**100
        1.35367986679107295756e9
        >>> 2*x
        2.46799999999999997158
        >>> x*2.0
        2.46799999999999997158
        >>> x.order()
        Infinity
        >>> x.prec()
        64
        >>> float(x)
        1.234
        >>> int(y)
        -1
    """
    def __init__(self, value, prec = 0):
        element.Element.__init__(self, real_field.RR_mpf)
        RealNumber.__init__(self, RealNumber_mpf)
        try:
            if isinstance(value, RealNumber_mpf):
                self.value = gmpy.mpf(value.value, prec)
            elif isinstance(value, RealNumber_decimal):
                self.value = gmpy.mpf(str(value.value), prec)
            elif isinstance(value, rational.Rational):
                self.value = gmpy.mpf(str(value.numerator()), prec) / \
                                gmpy.mpf(str(value.denominator()), prec)
                return
            elif isinstance(value, integer.Integer):
                self.value = gmpy.mpf(str(value))
                return
            elif isinstance(value, str):
                i = value.find("/")
                if i != -1:
                    self.value = gmpy.mpf(value[:i], prec) / \
                                    gmpy.mpf(value[i+1:], prec)
                    return
                self.value = gmpy.mpf(value, prec)
                return
            else:
                self.value = gmpy.mpf(value, prec)
        except TypeError:
            raise TypeError, "unable to coerce %s to a RealNumber"%value

    def _new(self, value, prec=0):
        return RealNumber_mpf(value, prec)

    def sign(self):
        if self < self.parent()(0):
            return -1
        else:
            return 1

    def sqrt(self):
        return RealNumber_mpf(self.value.sqrt(), self.prec())
  
    def prec(self):
        return self.value.getprec()
    
        
# doctests
if __name__ ==  '__main__':
    import doctest, sys
    from real_field import RealField_mpf, RealField_decimal
    import sage.rings.real_number as real_number
    doctest.testmod(sys.modules[__name__])

==> sage-v0.3-2005-05-09-src.tar: sage-0.3/sage/rings/real_number.py <==
"""
Real number

Real numbers in SAGE are represented either as GMP MPF's or 
using the Python Decimal type.  
"""


import gmpy
import math

import coerce
import decimal
import element
import infinity
import sage.misc.misc as misc
import operator
import sage.pari.wrap
import sage.rings.rational as rational
import sage.rings.integer as integer
import real_field
import sage.rings.polynomial_ring
import integer_ring

class RealNumber(element.Element):
    """
    Abstract real number base class.
    """
    def __init__(self, cls):
        self.__cls = cls

    def __abs__(self):
        if self < 0:
            return -self
        return self

    def __add__(self, right):
        """
        Return self + right.
        EXAMPLES:
            >>> RealNumber_mpf('2/3') + RealNumber_decimal(0.333)
            0.999666666666666666667
        """
        if not isinstance(right, self.__cls):
            return coerce.bin_op(self, right, operator.add)
        return self._new(self.value + right.value)
    
    def __cmp__(self, other):
        """
        Compare self and other.
        
        EXAMPLES:
        In comparisons numeric types, e.g., int, long, float, and those derived
        from Element, are converted to a RealNumber (if it is on the left), 
        to the precision of the number on the left.
            >>> R = RealNumber_mpf
            >>> R('2/3') < 1
            True
            >>> R('1/3') > 0.1
            True
            >>> R('1/3') > RealNumber_decimal(0.1)
            True
            >>> R('1/3') == R('0.3333333333333333')
            False
            
        Strings are not numeric, so they are not converted.
            >>> R(1) == '1'
            False
        """
        if not isinstance(other, self.__cls):
            if isinstance(other, (int, long, float, complex, element.Element)):
                other = self._new(other, self.prec())
            else:
                return -1
        return misc.generic_cmp(self.value, other.value)

    def __div__(self, right):
        """
        Return the quotient self / right.
        
        EXAMPLES:
            >>> R = RealField_mpf()
            >>> R(1) / R(3)
            0.333333333333333333333
        """
        if not isinstance(right, self.__cls):
            return coerce.bin_op(self, right, operator.div)
        return self._new(self.value / right.value)
        
    def __float__(self):
        """
        Return the coercion of self to a Python float.
        
        EXAMPLES:
            >>> R = RealField_mpf()
            
            >> pi = R.pi(200); pi
            3.14159265358979323846264338327950288419716939937510582097494459230782
            >> float(pi)
            3.1415926535897931
        """
        return float(self.value)
    
    def __int__(self):
        """
        Return the coercion of self to a Python int.  This is the integer
        part of self.
        
        EXAMPLES:
            >>> R = RealField_mpf()
            
            >> pi = R.pi(200); pi
            3.14159265358979323846264338327950288419716939937510582097494459230782
            >> int(pi)
            3
            
        If the real number is too large to fit in a Python int, it is 
        coerced to a Python long (note the L below):
            >>> int(R(141592653589793238462643383279502))
            141592653589793238462643383279502L
            
        Note that int is not "rounding".  It simply takes the integer part
        of the decimal expansion, including the sign:
            >>> int(R(-0.8))
            0
            >>> int(R(0.8))
            0
            >>> int(R(-2.3))
            -2
        """
        try:
            return int(self.value)
        except OverflowError:
            return long(self.value)
            
    def __invert__(self):
        """
        Return the inverse of self.
        
        EXAMPLES:
            >>> R = RealField_mpf()
            
            >> pi = R.pi(); pi
            3.14159265358979323846
            >> ~pi    # the additive inverse
            0.318309886183790671538
            
        Division by zero raises a ....
            >>> ~R(0)
            Traceback (most recent call last):
            ...
            ZeroDivisionError: inverse of 0 not defined
        """
        try:
            return self._new(1/self.value)
        except ZeroDivisionError:
            raise ZeroDivisionError, "inverse of 0 not defined"
            
    def __long__(self):
        """
        Return the coercion of self to a Python long.
        
        EXAMPLES:
            >>> R = RealField_mpf()
            
            >> pi = R.pi(200); pi
            3.14159265358979323846264338327950288419716939937510582097494459230782
            >> long(pi)
            3L
            
        Note that int is not "rounding".  It simply takes the integer part
        of the decimal expansion, including the sign:
            >>> long(R(-0.8))
            0L
            >>> long(R(0.8))
            0L
            >>> long(R(-2.3))
            -2L
        """
        return long(self.value)
    
    
    def __mul__(self, right):
        """
        Return the product of self and right.
        
        If right is not a RealNumber, an attempt is made to coerce self
        and right into a common parent.
        
        EXAMPLES:
            >>> R = RealField_mpf()
            >>> pi = R.pi()
            >>> A = pi * 25; A
            78.5398163397448309615
            >>> isinstance(A, real_number.RealNumber_mpf)
            True

        In this example, a string is automatically coerced to a real number.
            >>> pi * '2.01'
            6.31460123371548440927
            
        Of course two strings don't coerce to RealNumber's:
            >>> '3.14' * '2.01'
            Traceback (most recent call last):
            ...
            TypeError: can't multiply sequence by non-int
        """
        if not isinstance(right, self.__cls):
            return coerce.bin_op(self, right, operator.mul)
        return self._new(self.value * right.value)
    
    def __neg__(self):
        """
        Return the negative of self.
        
        EXAMPLES:
            >>> x = RealNumber_mpf('1.23456')
            >>> -x
            -1.23456
        """
        return self._new(-self.value)
    
    def __repr__(self):
        return "%s"%self.value


    def __radd__(self, left):
        return  self._new(left) + self
    
    def __sub__(self, right):
        if not isinstance(right, self.__cls):
            return coerce.bin_op(self, right, operator.sub)
        return self._new(self.value - right.value)
    
    def __rsub__(self, left):
        return self._new(left) - self

    def __rmul__(self, left):
        return self._new(left)*self
    
    def __rdiv__(self, left):
        return self._new(left)/self
    
    def __pow__(self, right):
        right = self.parent()(right)
        return self._new(self.value ** right.value)

    def __pos__(self):
        return self
  
    def _new(self, value, prec=0):
        raise NotImplementedError

    def arg(self):
        return 0

    def real(self):
        return self

    def imag(self):
        return self.parent()(0)

    def log(self):
        """
        Return the logarithm of self.

        IMPORTANT WARNING: Currently the computation is done by
        converting the number to a float and using the standard
        library function.  Thus there is precision loss!
        """
        z = math.log(float(self.value))
        return self.parent()(z)
  
    def order(self):
        if self.value == 1:
            return 1
        if self.value == -1:
            return 2
        return infinity.infinity
        
    def prec(self):
        raise NotImplementedError

    def __get_pari_object(self):
        try:
            return self.__pari_object
        except AttributeError:
            self.__pari_object = sage.pari.wrap.PariObject(str(self))
            return self.__pari_object

    pari_object = property(__get_pari_object, \
                           doc = "PARI real associated to this real number")

    def algebraic_dependence(self, n, flag=0):
        """
        Try to find a polynomial of degree at most n having this
        real number x as an approximate root.

        The polynomial that is obtained is not necessarily
        "correct", and it's not even guaranteed to be irreducible.
        One can check the closeness either by evaluation or by
        computing the roots of the polynomial given by algdep.

        INPUT:
            n -- positive integer
            flag -- 0 (default): Use a variant of the LLL algorithm
                       due to Hastard, Lagarias, and Schnorr (STACS
                       1986).  If the precision is too low, the
                       routine may enter an infinite loop.
                    >0: Use a standard LLL algorithm, where the
                       flag is a precision, which should be between
                       0.5 and 1.0 times the number of decimal digits
                       to which x was computed.
        OUTPUT:
            A polynomial over the integers.

        IMPLEMENTATION: Call PARI's algdep function.  Note that the
            default real precision of PARI is used, not the precision
            of this real number.

        EXAMPLE:
        We compute the minimal polynomial of sqrt(2):
        
            >>> import real_field, math
            >>> R = real_field.RealField()
            >>> a = R(math.sqrt(2))
            >>> a.algebraic_dependence(2)
            x^2 - 2

        Here we compute the real number 123/456, which is not in lowest terms,
        and find the minimal polynomial, which is a polynomial of degree 1.
            >>> a = R('123/456')
            >>> a
            0.269736842105263157895
            >>> a.algebraic_dependence(1)
            152*x - 41
            >>> 41/152.0
            0.26973684210526316
        """
        f = self.pari_object.algdep(n, flag)
        v = f.Vec().list()
        v.reverse()
        R = sage.rings.polynomial_ring.PolynomialRing(integer_ring.IntegerRing())
        g = R(v)
        return g
  
  
class RealNumber_decimal(RealNumber):
    """
    A real number with an arbitrary number of bits of precision.
    This is implemented using the builtin Python Decimal type.

    Create a number using
        RealNumber_decimal(value,)

    INPUT:
        value -- a float, integer, or string
    EXAMPLES:
        >>> R = real_number.RealNumber_decimal
        >>> R(1.32)
        1.32
        >>> R('2/3')
        0.6666666666666666666666666667
        >>> x = R(1.234); y = R(-1.1)
        >>> x < y
        False
        >>> x > y
        True
        >>> x + y
        0.134
        >>> x - y
        2.334
        >>> x * y
        -1.3574
        >>> x / y
        -1.121818181818181818181818182
        >>> x*100
        123.400
        >>> x**100
        1353679866.791074518495392801
        >>> 2*x
        2.468
        >>> x*2.0
        2.4680
        >>> x.order()
        Infinity
        >>> x.prec()
        Infinity
        >>> float(x)
        1.234
        >>> int(y)
        -1
    """
    def __init__(self, value):
        element.Element.__init__(self, real_field.RR_decimal)
        RealNumber.__init__(self, RealNumber_decimal)
        try:
            if isinstance(value, decimal.Decimal):
                self.value = decimal.Decimal(value)
            elif isinstance(value, RealNumber_decimal):
                self.value = decimal.Decimal(value.value)
            elif isinstance(value, rational.Rational):
                self.value = decimal.Decimal(str(value.numerator())) / \
                                decimal.Decimal(str(value.denominator()))
                return
            elif isinstance(value, str):
                i = value.find("/")
                if i != -1:
                    self.value = decimal.Decimal(value[:i]) / \
                                    decimal.Decimal(value[i+1:])
                    return
                self.value = decimal.Decimal(value)
                return
            else:
                self.value = decimal.Decimal(str(value))
        except TypeError:
            raise TypeError, "unable to coerce %s to a RealNumber_decimal"%value
        
    def _new(self, value, prec=0):
        return RealNumber_decimal(value)
  
    def prec(self):
        return infinity.infinity
    
 
class RealNumber_mpf(RealNumber):
    """
    A real number with a fixed number of bits of precision.

    Create a number using
         RealNumber_mpf(value, bits=0)

    INPUT:
        value -- a float, integer, or string
        prec -- an int, lower bound on the number of bits of precision
                (always >= 64).
    EXAMPLES:
        >>> R = real_number.RealNumber_mpf
        >>> x = R(1.234); y = R(-1.1)
        >>> x < y
        False
        >>> x > y
        True
        >>> x + y
        0.133999999999999896971
        >>> x - y
        2.33400000000000007461
        >>> x * y
        -1.35740000000000009397
        >>> x / y
        -1.12181818181818171468
        >>> x*100
        123.399999999999998579
        >>> x**100
        1.35367986679107295756e9
        >>> 2*x
        2.46799999999999997158
        >>> x*2.0
        2.46799999999999997158
        >>> x.order()
        Infinity
        >>> x.prec()
        64
        >>> float(x)
        1.234
        >>> int(y)
        -1
    """
    def __init__(self, value, prec = 0):
        element.Element.__init__(self, real_field.RR_mpf)
        RealNumber.__init__(self, RealNumber_mpf)
        try:
            if isinstance(value, RealNumber_mpf):
                self.value = gmpy.mpf(value.value, prec)
            elif isinstance(value, RealNumber_decimal):
                self.value = gmpy.mpf(str(value.value), prec)
            elif isinstance(value, rational.Rational):
                self.value = gmpy.mpf(str(value.numerator()), prec) / \
                                gmpy.mpf(str(value.denominator()), prec)
                return
            elif isinstance(value, integer.Integer):
                self.value = gmpy.mpf(str(value))
                return
            elif isinstance(value, str):
                i = value.find("/")
                if i != -1:
                    self.value = gmpy.mpf(value[:i], prec) / \
                                    gmpy.mpf(value[i+1:], prec)
                    return
                self.value = gmpy.mpf(value, prec)
                return
            else:
                self.value = gmpy.mpf(value, prec)
        except TypeError:
            raise TypeError, "unable to coerce %s to a RealNumber_mpf"%value

    def _new(self, value, prec=0):
        return RealNumber_mpf(value, prec)

    def sign(self):
        if self < self.parent()(0):
            return -1
        else:
            return 1

    def sqrt(self):
        return RealNumber_mpf(self.value.sqrt(), self.prec())
  
    def prec(self):
        return self.value.getprec()
    
        
# doctests
if __name__ ==  '__main__':
    import doctest, sys
    from real_field import RealField_mpf, RealField_decimal
    import sage.rings.real_number as real_number
    doctest.testmod(sys.modules[__name__])
